#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # Database Schemas
# 
# Material for the hackathon ENSAE / Red Cross / 2015. Schema of the databases.

# In[1]:


get_ipython().run_line_magic('matplotlib', 'inline')
import matplotlib.pyplot as plt
plt.style.use('ggplot')


# In[2]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# ## Connection to the cluster
# 
# You can set your credentials and retrieve them with module [keyring](https://pypi.python.org/pypi/keyring).

# In[3]:


# don't leave these line in the notebook.
# from pyquickhelper.loghelper import get_password
# set_password("CRCREDENTIALS", "blobstorage", "...")
# set_password("CRCREDENTIALS", "password", "...")


# In[4]:


from pyquickhelper.loghelper import get_password
hackathon = {}
hackathon["blob_storage"] = get_password("CRCREDENTIALS", "blobstorage")
hackathon["password"] = get_password("CRCREDENTIALS", "password")


# In[5]:


blobstorage = hackathon["blob_storage"]
blobpassword = hackathon["password"]


# In[6]:


get_ipython().run_line_magic('load_ext', 'pyensae')


# In[7]:


cl, bs = get_ipython().run_line_magic('blob_open', '')
cl, bs


# ## Variables and tables

# In[8]:


get_ipython().run_line_magic('blob_down', 'croix-rouge/data/ITMMASTER.schema.txt ITMMASTER.schema.txt')
get_ipython().run_line_magic('blob_down', 'croix-rouge/data/SINVOICE.schema.txt SINVOICE.schema.txt')
get_ipython().run_line_magic('blob_down', 'croix-rouge/data/SINVOICEV.schema.txt SINVOICEV.schema.txt')


# ### ITMMASTER

# In[9]:


get_ipython().run_line_magic('blob_ls', 'croix-rouge')


# In[10]:


df = get_ipython().run_line_magic('blob_head', 'croix-rouge/data/ITMMASTER.schema.txt')
df.head()


# In[11]:


from pyquickhelper.pandashelper import df2rst
with open("sch_itmmaster.txt", "w", encoding="utf8") as f:
    dfi = df.reset_index(drop=False)
    dfi["index"] = dfi["index"]+1
    f.write(df2rst(dfi.fillna("")))


# ### SINVOICE

# In[12]:


from ensae_projects.datainc.croix_rouge import get_meaning
meaning = get_meaning("invoice")
meaning.head()


# In[13]:


meaning.columns


# In[14]:


df = get_ipython().run_line_magic('blob_head', 'croix-rouge/data/SINVOICE.schema.txt')
df.head()


# In[15]:


merge = df.merge(meaning, right_on=meaning.columns[0], left_on="Zone", how="outer")
print(merge.shape, df.shape, meaning.shape)
merge = merge[~merge["Zone"].isnull()]
print(merge.shape)
merge.head(n=2)


# In[16]:


from pyquickhelper.pandashelper import df2rst
with open("sch_invoice.txt", "w", encoding="utf8") as f:
    dfi = merge.reset_index(drop=False)
    dfi["index"] = dfi["index"]+1
    f.write(df2rst(dfi.fillna("")))


# ### INVOICE_V

# In[17]:


df = get_ipython().run_line_magic('blob_head', 'croix-rouge/data/SINVOICEV.schema.txt')
df.head(n=2)


# In[18]:


mergev = df.merge(meaning, right_on=meaning.columns[0], left_on="Zone", how="outer")
print(mergev.shape, df.shape, meaning.shape)
mergev = mergev[~mergev["Zone"].isnull()]
print(mergev.shape)
mergev.head(n=2)


# In[19]:


from pyquickhelper.pandashelper import df2rst
with open("sch_invoice_v.txt", "w", encoding="utf8") as f:
    dfi = mergev.reset_index(drop=False)
    dfi["index"] = dfi["index"]+1
    f.write(df2rst(dfi.fillna("")))


# Differences between ``INVOICE_V`` and ``INVOICE``

# In[20]:


merge["table"] = "INVOICE"
mergev["table"] = "INVOICE_V"
diff = merge[["Zone", "table"]].merge(mergev[["Zone", "table"]], how="outer", on="Zone", suffixes=("", "_V")).copy()
diff["COMMON"] = True
diff.loc[diff["table"].isnull() | diff["table_V"].isnull(), "COMMON"] = False
diff = diff.sort_values(["COMMON", "Zone"])
diff.shape


# In[21]:


from pyquickhelper.pandashelper import df2rst
with open("sch_invoice_diff.txt", "w", encoding="utf8") as f:
    dfi = diff.reset_index(drop=False)
    dfi["index"] = dfi["index"]+1
    f.write(df2rst(dfi.fillna("")))


# ### stojou

# In[22]:


df = cl.df_head(bs, "croix-rouge", "data/stojou.csv", as_df=False, stop_at=2000)
res = df.split("\r")[0]
res


# In[23]:


import pandas
stojou = pandas.DataFrame(data={ "Zone":[ _.replace("_0", "") for _ in res.split(",")]})
stojou.to_csv("stojou.schema.txt")
stojou["table_SJ"] = "stojou"
stojou.head()


# In[24]:


diff.head()


# In[25]:


final = diff[["Zone", "table", "table_V"]].merge(stojou, how="outer", on="Zone", suffixes=("", "_STOJOU")).copy()
final = final.sort_values(["table_SJ", "Zone"])
final.shape


# In[26]:


from pyquickhelper.pandashelper import df2rst
with open("sch_stojou.txt", "w", encoding="utf8") as f:
    dfi = final.reset_index(drop=False)
    dfi["index"] = dfi["index"]+1
    f.write(df2rst(dfi.fillna("")))


# ## Overview of the data

# In[27]:


df = get_ipython().run_line_magic('blob_head', 'croix-rouge/build/SINVOICE_M.csv --sep=,')
df.head()


# In[28]:


df.columns


# ## Header for the dataframe
# 
# Just to check that suffix ``_0`` is implicit in the documentation.

# In[29]:


get_ipython().run_line_magic('blob_ls', 'croix-rouge')


# In[30]:


df = cl.df_head(bs, "croix-rouge", "data/stojou.csv", as_df=False, stop_at=2000)
df.split("\r")[0]


# In[31]:


df = cl.df_head(bs, "croix-rouge", "data/SINVOICEV.txt", as_df=False, stop_at=2000)
df.split("\r")[0]


# In[32]:


df = cl.df_head(bs, "croix-rouge", "data/SINVOICE.txt", as_df=False, stop_at=2000)
df.split("\r")[0]


# In[33]:


df = cl.df_head(bs, "croix-rouge", "data/ITMMASTER.txt", as_df=False, stop_at=2000)
df.split("\r")[0]


# In[34]:


df = cl.df_head(bs, "croix-rouge", "data/enseignes_france.csv", as_df=True, stop_at=None)
df.head()


# In[35]:


df.columns


# In[36]:


df = cl.df_head(bs, "croix-rouge", "build/Test_CRFFOR.GACCTMPD.csv", as_df=False, stop_at=2000)
df.split("\n")[0]


# ## Close connection

# In[37]:


get_ipython().run_line_magic('blob_close', '')


# ## Use module ensae_projects
# 
# Module [ensae_projects](http://www.xavierdupre.fr/app/ensae_projects/helpsphinx/index.html) includes helpers for this event. Here is how to get the joined schemas for all tables in one Excel file. Some data is not included in the module but it is encrypted. You need a password given to any participant. You can store it in environment variable ``PWDCROIXROUGE`` to avoid typing it each time you need it.

# ### joined schemas

# In[38]:


from ensae_projects.datainc.croix_rouge import merge_schema
df = merge_schema()


# In[39]:


df.head(n=1)


# In[40]:


df.to_excel("joined_schemas.xlsx")


# ### specific schemas

# In[41]:


from ensae_projects.datainc.croix_rouge import get_meaning
df = get_meaning("ITMMASTER")
df.head(n=2)


# In[42]:


from ensae_projects.datainc.croix_rouge import get_meaning
df = get_meaning("SINVOICE")
df.head(n=2)


# In[43]:


from ensae_projects.datainc.croix_rouge import get_meaning
df = get_meaning("SINVOICE_V")
df.head(n=2)


# In[44]: